// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS2AO2A2M/utils/bdo_io_mode.c $
// $Rev: 45184 $
// $Date: 2019-07-01 16:59:04 -0500 (Mon, 01 Jul 2019) $

// 16AISS2AO2A2M: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss2ao2a2m_bdo_io_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS2AO2A2M_IOCTL_BDO_IO_MODE service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aiss2ao2a2m_bdo_io_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			ret;

	if (verbose)
		gsc_label_index("BDO I/O Mode", index);

	ret		= aiss2ao2a2m_ioctl(fd, AISS2AO2A2M_IOCTL_BDO_IO_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this board.";
			break;

		case GSC_IO_MODE_PIO:

			ptr	= "PIO";
			break;

		case GSC_IO_MODE_BMDMA:

			ptr	= "Block Mode DMA not supported";
			break;

		case GSC_IO_MODE_DMDMA:

			ptr	= "Demand Mode DMA not supported";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


